@@ -8,6 +8,9 @@ Required packages:
 - numpy
 - PIL
 
+Warning:
+- There is a memory error in numpy 1.6.0 (ticket #1896) that this code triggers.
+
 Mouse Gestures
     Zoom (Z mode or use button 3):
     - Drag UL->LR to zoom in on the enclosed region.
@@ -151,6 +154,10 @@ History:
 2009-11-04 ROwen    Added ann_Line and ann_Text annotations.
                     Added cnvOffset argument to Annotation and GrayImageWdg.addAnnotation.
 2010-10-19 ROwen    Made value a float instead of an int to handle float images better, including NaN.
+2011-07-06 ROwen    Made scaling functions operate on scaledArr in place.
+                    Added doRescale argument to redisplay to reduce needless recomputation of scaledArr.
+2011-07-27 ROwen    Removed doRescale argument from redisplay because it was causing bugs.
+                    Simplified the code for reusing scaledArr.
 """
 import weakref
 import Tkinter
@@ -1061,21 +1068,24 @@ class GrayImageWdg(Tkinter.Frame, RO.AddCallback.BaseMixin):
         """
         if self.dataArr == None:
             return
-        
+            
         self.strMsgWdg.grid_remove()
 
         try:
             dataShapeXY = self.dataArr.shape[::-1]
             
             # offset so minimum display value = scaling function minimum input
-            # note: this form of equation reuses the input array for output
-            numpy.subtract(self.dataArr, float(self.dataDispMin), self.scaledArr)
+            # reuse existing scaledArr memory if possible
+            if self.scaledArr == None or self.scaledArr.shape != self.dataArr.shape:
+                self.scaledArr = self.dataArr - float(self.dataDispMin)
+            else:
+                numpy.subtract(self.dataArr, float(self.dataDispMin), self.scaledArr)
             
             offsetDispRange = [0.0, float(self.dataDispMax - self.dataDispMin)]
             
             # apply scaling function, if any
             if self.scaleFunc:
-                self.scaledArr = self.scaleFunc(self.scaledArr).astype(numpy.float32)
+                self.scaleFunc(self.scaledArr, self.scaledArr)
                 scaledMin, scaledMax = self.scaleFunc(offsetDispRange).astype(float)
             else:
                 scaledMin, scaledMax = offsetDispRange
@@ -1164,8 +1174,8 @@ class GrayImageWdg(Tkinter.Frame, RO.AddCallback.BaseMixin):
         I don't yet set m. I might be able to add it, I'm just not sure
         it's useful given how I already stretch data
         """
-        def arcsinh(x):
-            return numpy.arcsinh(numpy.multiply(x, scaleFac))
+        def arcsinh(x, out=None):
+            return numpy.arcsinh(numpy.multiply(x, scaleFac), out)
         self.setScaleFunc(arcsinh)
     
     def scaleLinear(self):
@@ -1265,14 +1275,9 @@ class GrayImageWdg(Tkinter.Frame, RO.AddCallback.BaseMixin):
             self.sortedData = unmaskedArr
             self.sortedData.sort()
     
-            # scaledArr gets computed in place by redisplay;
-            # for now just allocate space of the appropriate type
-            self.scaledArr = numpy.zeros(self.dataArr.shape, dtype=numpy.float32)
-            
             # look for data leaks
             self._trackMem(self.dataArr, "dataArr")
             self._trackMem(self.sortedData, "sortedData")
-            self._trackMem(self.scaledArr, "scaledArr")
             
             self.doRangeMenu(redisplay=False)
             
